;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; DesktopStyle
; Switching the desktop's icon presentation
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.386
.MODEL FLAT, STDCALL
OPTION CASEMAP:NONE


;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
;                                  I N C L U D E   F I L E S
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

INCLUDE \masm32\include\windows.inc
INCLUDE \masm32\include\kernel32.inc
INCLUDE \masm32\include\user32.inc

INCLUDELIB \masm32\lib\kernel32.lib
INCLUDELIB \masm32\lib\user32.lib


;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
;                                         C O D E
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.CODE

szProgman        db "Progman",0
szDefView        db "SHELLDLL_DefView",0
szListView       db "SysListView32",0
szHeader         db "SysHeader32",0

;
; GetLVHandle
; Retrieve the ListView handle
;
; >= N/A
; => eax = Handle (or NULL if failure)
;

GetLVHandle PROC
  invoke  GetDesktopWindow
  test    eax, eax
  jz      @F
  invoke  FindWindowEx, eax, 0, ADDR szProgman, 0
  test    eax, eax
  jz      @F
  invoke  FindWindowEx, eax, 0, ADDR szDefView, 0
  test    eax, eax
  jz      @F
  invoke  FindWindowEx, eax, 0, ADDR szListView, 0
@@:
  ret
GetLVHandle ENDP

;
; ChangeLVStyle
; Change the ListView style
;
; >= lvStyle = LVS_LIST / LVS_REPORT / LVS_ICON / LVS_SMALLICON
; => eax = 0 if failure, nonzero if success
;

ChangeLVStyle PROC lvStyle:DWORD
  LOCAL   hWnd:DWORD
  LOCAL   hHeaderHack:DWORD
  LOCAL   rRect:RECT

  mov     hHeaderHack, 0
  invoke  GetLVHandle
  test    eax, eax
  jnz     @F
  ret
@@:
  mov     hWnd, eax

  invoke  GetWindowLong, eax, GWL_STYLE
  and     eax, (NOT LVS_TYPEMASK) AND (NOT LVS_NOSCROLL)

  ;-----------------------------------------------------------------------------
  ; LVS_LIST
  ;
  cmp     lvStyle, LVS_LIST
  jne     @CLVS_CheckReport

  or      eax, LVS_LIST
  jmp     @CLVS_ChangeStyle

@CLVS_CheckReport:
  ;-----------------------------------------------------------------------------
  ; LVS_REPORT
  ;
  cmp     lvStyle, LVS_REPORT
  jne     @CLVS_CheckIcon

  or      eax, LVS_REPORT
  push    eax

  invoke  FindWindowEx, hWnd, 0, ADDR szHeader, 0
  test    eax, eax
  jnz     @F
  mov     hHeaderHack, 1
@@:
  pop     eax
  jmp     @CLVS_ChangeStyle

@CLVS_CheckIcon:
  ;-----------------------------------------------------------------------------
  ; LVS_ICON
  ;
  cmp     lvStyle, LVS_ICON
  jne     @CLVS_CheckSmallIcon

  or      eax, LVS_ICON OR LVS_NOSCROLL
  jmp     @CLVS_ChangeStyle

@CLVS_CheckSmallIcon:
  ;-----------------------------------------------------------------------------
  ; LVS_SMALLICON
  ;
  cmp     lvStyle, LVS_SMALLICON
  jne     @CLVS_BadArgument

  or      eax, LVS_SMALLICON OR LVS_NOSCROLL
  jmp     @CLVS_ChangeStyle

@CLVS_BadArgument:
  ;-----------------------------------------------------------------------------
  ; Bad argument
  ;
  xor     eax, eax
  ret

@CLVS_ChangeStyle:
  ;-----------------------------------------------------------------------------
  ; Change style
  ; >= eax = New style
  ;
  invoke  SetWindowLong, hWnd, GWL_STYLE, eax

  cmp     hHeaderHack, 0
  je      @CLVS_Exit

  ;-----------------------------------------------------------------------------
  ; Right click hack for LVS_REPORT
  ;

  invoke  FindWindowEx, hWnd, 0, ADDR szHeader, 0
  test    eax, eax
  jz      @CLVS_Exit
  mov     hHeaderHack, eax

  invoke  GetClientRect, hHeaderHack, ADDR rRect
  test    eax, eax
  jz      @CLVS_Exit

  mov     eax, rRect.bottom
  shl     eax, 16
  mov     ax, WORD PTR rRect.left
  push    eax
  add     ax, 1
  push    eax

  invoke  PostMessage, hWnd, WM_RBUTTONDOWN, MK_RBUTTON, eax
  pop     eax
  invoke  PostMessage, hWnd, WM_RBUTTONUP, 0, eax
  mov     eax, [esp]
  invoke  PostMessage, hWnd, WM_LBUTTONDOWN, MK_LBUTTON, eax
  pop     eax
  invoke  PostMessage, hWnd, WM_LBUTTONUP, 0, eax

@CLVS_Exit:
  mov     eax, 1
  ret
ChangeLVStyle ENDP

;
; EntryPoint
;

EntryPoint:
  ; Use LVS_LIST / LVS_REPORT / LVS_ICON / LVS_SMALLICON
  ; Any other value is ignored
  invoke  ChangeLVStyle, LVS_REPORT

  invoke  ExitProcess, 0


;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
END EntryPoint
